package com.gic.store.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.store.constant.StoreFieldDateEnum;
import com.gic.store.constant.StoreFieldTextEnum;
import com.gic.store.constant.StoreFieldTypeEnum;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.dto.StoreFieldSelectDTO;
import com.gic.store.dto.StoreRegionDTO;
import com.gic.store.service.StoreFieldApiService;
import com.gic.store.service.StoreFieldSelectApiService;
import com.gic.store.service.StoreRegionApiService;
import com.gic.store.utils.ErrorCode;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.qo.storefield.StoreFieldQO;
import com.gic.store.web.qo.storefield.StoreFieldSelectQO;
import com.gic.store.web.vo.storefield.StoreFieldEditVO;
import com.gic.store.web.vo.storefield.StoreFieldVO;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author guojx
 * @date 2019/6/28 2:01 PM
 */
@RestController
@RequestMapping("/store-field")
public class StoreFieldController {

    private static final Logger logger = LoggerFactory.getLogger(StoreFieldController.class);

    @Autowired
    private StoreFieldApiService storeFieldApiService;
    @Autowired
    private StoreFieldSelectApiService storeFieldSelectApiService;

    @Autowired
    private StoreRegionApiService storeRegionApiService;

    @RequestMapping("/list-store-field-type")
    public RestResponse listStoreFieldType() {
        Map<Integer, String> result = new HashMap<>(16);
        for (StoreFieldTypeEnum typeEnum : StoreFieldTypeEnum.values()) {
            result.put(typeEnum.getCode(), typeEnum.getMessage());
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/list-store-field-date-type")
    public RestResponse listStoreFieldDateType() {
        Map<Integer, String> result = new HashMap<>(16);
        for (StoreFieldDateEnum typeEnum : StoreFieldDateEnum.values()) {
            result.put(typeEnum.getCode(), typeEnum.getMessage());
        }
        return RestResponse.success(result);
    }

    /**
     * 文本类型
     * @return
     */
    @RequestMapping("/list-store-field-text-type")
    public RestResponse listStoreFieldTextType() {
        Map<Integer, String> result = new HashMap<>(16);
        for (StoreFieldTextEnum typeEnum : StoreFieldTextEnum.values()) {
            result.put(typeEnum.getCode(), typeEnum.getMessage());
        }
        return RestResponse.success(result);
    }

    @RequestMapping("/save-store-field")
    public RestResponse save(@Validated({StoreFieldQO.SaveValidView.class}) StoreFieldQO storeFieldQO) {
        String regions = storeFieldQO.getStoreRegionIds();
        String[] regionArr = regions.split(",");
        StringBuilder region = new StringBuilder("_");
        for (String str : regionArr) {
            region.append(Integer.parseInt(str));
            region.append("_");
        }
        storeFieldQO.setStoreRegionId(region.toString());
        ServiceResponse result = storeFieldApiService.save(transferQoToDTO(storeFieldQO));
        if (!result.isSuccess()) {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
        return RestResponse.success(result.getResult());
    }

    @RequestMapping("/edit-store-field")
    public RestResponse edit(@Validated({StoreFieldQO.SaveValidView.class, StoreFieldQO.RemoveValidView.class}) StoreFieldQO storeFieldQO) {
        String regions = storeFieldQO.getStoreRegionIds();
        String[] regionArr = regions.split(",");
        StringBuilder region = new StringBuilder("_");
        for (String str : regionArr) {
            region.append(str);
            region.append("_");
        }
        storeFieldQO.setStoreRegionId(region.toString());
        ServiceResponse result = storeFieldApiService.edit(transferQoToDTO(storeFieldQO));
        if (!result.isSuccess()) {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
        return RestResponse.success(result.getResult());
    }

    @RequestMapping("/get-store-field")
    public RestResponse getStoreField(@Validated({StoreFieldQO.RemoveValidView.class}) StoreFieldQO storeFieldQO) {
        ServiceResponse<StoreFieldDTO> result = storeFieldApiService.getStoreField(storeFieldQO.getStoreFieldId());
        if (result.isSuccess()) {
            StoreFieldEditVO vo = EntityUtil.changeEntityNew(StoreFieldEditVO.class, result.getResult());
            String regionId = result.getResult().getStoreRegionId();
            String tempRegion = regionId.replaceAll("_", ",");
            vo.setStoreRegionIds(tempRegion.substring(1, tempRegion.length() - 1));
            return RestResponse.success(vo);
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/remove-store-field")
    public RestResponse removeStoreField(@Validated({StoreFieldQO.RemoveValidView.class}) StoreFieldQO storeFieldQO) {
        ServiceResponse result = storeFieldApiService.delete(storeFieldQO.getStoreFieldId());
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/remove-store-field-batch")
    public RestResponse removeStoreFieldBatch(boolean isDeleteAll, String storeFieldIds) {
        if (isDeleteAll) {
            ServiceResponse result = storeFieldApiService.deleteAll(StoreGroupConstant.TEST_ENTERPRISE_ID);
            if (result.isSuccess()) {
                return RestResponse.success(result.getResult());
            } else {
                return RestResponse.failure(result.getCode(), result.getMessage());
            }
        } else {
            if (StringUtils.isBlank(storeFieldIds)) {
                return RestResponse.failure(ErrorCode.ERR_5.getCode(), "主键值不能为空");
            }
            String[] ids = storeFieldIds.split(",");
            List<Integer> storeFieldIdList = new ArrayList<>();
            for (String id : ids) {
                storeFieldIdList.add(Integer.parseInt(id));
            }
            ServiceResponse result = storeFieldApiService.deleteBatch(storeFieldIdList);
            if (result.isSuccess()) {
                return RestResponse.success(result.getResult());
            } else {
                return RestResponse.failure(result.getCode(), result.getMessage());
            }
        }

    }

    @RequestMapping("/list-store-field")
    public RestResponse listStoreField(String search, PageQO pageQO) {
        ServiceResponse<Page<StoreFieldDTO>> result = storeFieldApiService.listStoreField(StoreGroupConstant.TEST_ENTERPRISE_ID, search, pageQO.getCurrentPage(), pageQO.getPageSize());
        if (result.isSuccess()) {
            ServiceResponse<List<StoreRegionDTO>> serviceResponse = storeRegionApiService.listStoreRegion(StoreGroupConstant.TEST_ENTERPRISE_ID, null);
            List<StoreRegionDTO> regionDTOList = serviceResponse.getResult();
            Map<Integer, String> regionMap = new HashMap<>(16);
            for (StoreRegionDTO regionDTO : regionDTOList) {
                regionMap.put(regionDTO.getRegionId(), regionDTO.getRegionName());
            }

            //转 vo
            Page<StoreFieldDTO> resultPage = result.getResult();
            List<StoreFieldDTO> storeFieldDTOList = resultPage.getResult();
            List<StoreFieldVO> voList = EntityUtil.changeEntityListNew(StoreFieldVO.class, storeFieldDTOList);
            for (StoreFieldVO storeFieldVO : voList) {
                storeFieldVO.setRegionName(getRegionNameByRegionId(storeFieldVO.getStoreRegionId(), regionMap));
            }
            Page<StoreFieldVO> page = EntityUtil.changeEntityNew(Page.class, resultPage);
            page.setResult(voList);
            return RestResponse.success(page);
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/swap-store-field")
    public RestResponse swapStoreField(@Validated({StoreFieldQO.SwapValidView.class, StoreFieldQO.RemoveValidView.class}) StoreFieldQO storeFieldQO) {
        int upOrDown = storeFieldQO.getUpOrDown();
        ServiceResponse result;
        switch (upOrDown) {
            case StoreGroupConstant.SORT_UP:
               result = storeFieldApiService.upOrDown(storeFieldQO.getStoreFieldId(), true);
               break;
            case StoreGroupConstant.SORT_DOWN:
                result = storeFieldApiService.upOrDown(storeFieldQO.getStoreFieldId(), false);
                break;
            case StoreGroupConstant.SORT_BOTTOM:
                result = storeFieldApiService.sortBottom(storeFieldQO.getStoreFieldId());
                break;
            case StoreGroupConstant.SORT_TOPPING:
                result = storeFieldApiService.sortTopping(storeFieldQO.getStoreFieldId());
                break;
            default:
                result = null;
        }
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/set-store-field-sort-value")
    public RestResponse setStoreFieldSortValue(@Validated({StoreFieldQO.SetSortValueValidView.class, StoreFieldQO.RemoveValidView.class}) StoreFieldQO storeFieldQO) {
        ServiceResponse result = storeFieldApiService.setStoreFieldSort(storeFieldQO.getStoreFieldId(), storeFieldQO.getSortValue());
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/save-store-field-select")
    public RestResponse saveFieldValue(String storeFieldSelectNames, String storeFieldSelectCodes, Integer storeFieldId) {
        if (storeFieldId == null) {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), "自定义字段值不能为空");
        }
        if (validStoreFieldSelectSaveParam(storeFieldSelectNames, storeFieldSelectCodes)) {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), "参数错误（重复或者个数不对齐）");
        }
        String[] nameArr = storeFieldSelectNames.split(",");
        String[] codeArr = storeFieldSelectCodes.split(",");
        List<StoreFieldSelectDTO> list = new ArrayList<>(nameArr.length);
        for (int i = 0, length = nameArr.length; i < length; i ++) {
            StoreFieldSelectDTO dto = new StoreFieldSelectDTO();
            dto.setStoreFieldSelectName(nameArr[i]);
            dto.setStoreFieldSelectCode(codeArr[i]);
            dto.setStoreFieldId(storeFieldId);
            dto.setEnterpriseId(StoreGroupConstant.TEST_ENTERPRISE_ID);
            list.add(dto);
        }
        for (StoreFieldSelectDTO record : list) {
            storeFieldSelectApiService.save(record);
        }
        return RestResponse.success();
    }

    @RequestMapping("/edit-store-field-select")
    public RestResponse editStoreFieldSelect(@Validated({StoreFieldSelectQO.EditValidView.class}) StoreFieldSelectQO storeFieldSelectQO) {
        ServiceResponse result = storeFieldSelectApiService.edit(transferSelectQoToDTO(storeFieldSelectQO));
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/swap-store-field-select")
    public RestResponse swapStoreFieldSelect(@Validated({StoreFieldSelectQO.SwapValidView.class, StoreFieldSelectQO.RemoveValidView.class}) StoreFieldSelectQO storeFieldSelectQO) {
        int upOrDown = storeFieldSelectQO.getUpOrDown();
        ServiceResponse result;
        switch (upOrDown) {
            case StoreGroupConstant.SORT_UP:
                result = storeFieldSelectApiService.upOrDown(storeFieldSelectQO.getStoreFieldSelectId(), true);
                break;
            case StoreGroupConstant.SORT_DOWN:
                result = storeFieldSelectApiService.upOrDown(storeFieldSelectQO.getStoreFieldSelectId(), false);
                break;
            case StoreGroupConstant.SORT_BOTTOM:
                result = storeFieldSelectApiService.sortBottom(storeFieldSelectQO.getStoreFieldSelectId());
                break;
            case StoreGroupConstant.SORT_TOPPING:
                result = storeFieldSelectApiService.sortTopping(storeFieldSelectQO.getStoreFieldSelectId());
                break;
            default:
                result = null;
        }
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/list-store-field-select")
    public RestResponse listStoreFieldSelect(@Validated({StoreFieldSelectQO.ListValidView.class}) StoreFieldSelectQO storeFieldSelectQO, PageQO pageQO) {
        ServiceResponse<Page<StoreFieldSelectDTO>> result = storeFieldSelectApiService.listStoreFieldSelect(storeFieldSelectQO.getStoreFieldId(), storeFieldSelectQO.getSearch(),
                pageQO.getCurrentPage(), pageQO.getPageSize());
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/set-store-field-select-sort-value")
    public RestResponse setStoreFieldSelectSortValue(@Validated({StoreFieldSelectQO.SetSortValueValidView.class, StoreFieldSelectQO.RemoveValidView.class}) StoreFieldSelectQO storeFieldSelectQO) {
        ServiceResponse result = storeFieldSelectApiService.setStoreFieldSelectSort(storeFieldSelectQO.getStoreFieldSelectId(), storeFieldSelectQO.getSortValue());
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/delete-store-field-select")
    public RestResponse editStoreFieldSelect(String storeFieldSelectIds) {
        if (StringUtils.isBlank(storeFieldSelectIds)) {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), "主键不能为空");
        }
        String[] ids = storeFieldSelectIds.split(",");
        for (String str : ids) {
            ServiceResponse result = storeFieldSelectApiService.delete(Integer.parseInt(str));
            if (!result.isSuccess()) {
                return RestResponse.failure(result.getCode(), result.getMessage());
            }
        }
        return RestResponse.success();
    }

    private StoreFieldDTO transferQoToDTO(StoreFieldQO storeFieldQO) {
        StoreFieldDTO storeFieldDTO = EntityUtil.changeEntityNew(StoreFieldDTO.class, storeFieldQO);
        storeFieldDTO.setEnterpriseId(StoreGroupConstant.TEST_ENTERPRISE_ID);
        return storeFieldDTO;
    }

    private StoreFieldSelectDTO transferSelectQoToDTO(StoreFieldSelectQO storeFieldSelectQO) {
        StoreFieldSelectDTO storeFieldDTO = EntityUtil.changeEntityNew(StoreFieldSelectDTO.class, storeFieldSelectQO);
        storeFieldDTO.setEnterpriseId(StoreGroupConstant.TEST_ENTERPRISE_ID);
        return storeFieldDTO;
    }

    private String getRegionNameByRegionId(String regionId, Map<Integer, String> regionMap) {
        String[] regionIdArr = regionId.split("_");
        StringBuilder result = new StringBuilder();
        for (String str : regionIdArr) {
            if (StringUtils.isBlank(str)) {
                continue;
            }
            result.append(regionMap.get(Integer.parseInt(str)));
            result.append("/");
        }
        return result.toString().substring(0, result.toString().length() - 1);
    }

    private boolean validStoreFieldSelectSaveParam(String storeFieldSelectNames, String storeFieldSelectCodes) {
        if (StringUtils.isBlank(storeFieldSelectCodes)) {
            return true;
        }
        if (StringUtils.isBlank(storeFieldSelectNames)) {
            return true;
        }
        String[] nameArr = storeFieldSelectNames.split(",");
        String[] codeArr = storeFieldSelectCodes.split(",");
        if (nameArr.length != codeArr.length) {
            return true;
        }
        return isRepeatCode(codeArr) || isRepeatCode(nameArr);
    }

    private boolean isRepeatCode(String[] codeArr) {
        Map<String, String> tempMap = new HashMap<>(16);
        for (String code : codeArr) {
            if (tempMap.containsKey(code)) {
                //如果包含了，说明重复
                return true;
            }
            tempMap.put(code, "1");
        }
        return false;
    }
}
